package eu.jonahbauer.chat.server.management.impl;

import eu.jonahbauer.chat.bot.impl.BotCreationException;
import eu.jonahbauer.chat.bot.impl.ChatBotFactory;
import eu.jonahbauer.chat.server.bot.ChatBotSupervisor;
import eu.jonahbauer.chat.server.management.BotConfigSupport;
import eu.jonahbauer.chat.server.management.ChatBotManagerMXBean;
import lombok.SneakyThrows;
import org.jetbrains.annotations.NotNull;

import java.lang.management.ManagementFactory;
import java.util.*;
import java.util.stream.Collectors;

public enum ChatBotManager implements ChatBotManagerMXBean {
    INSTANCE;

    @SneakyThrows
    ChatBotManager() {
        var server = ManagementFactory.getPlatformMBeanServer();
        server.registerMBean(new AdvancedMBean(this), ChatBotManagerMXBean.NAME);
    }

    @Override
    public void start(@NotNull String name, @NotNull String type, @NotNull List<@NotNull String> channels) {
        try {
            ChatBotSupervisor.INSTANCE.start(name, type, channels);
        } catch (BotCreationException ex) {
            throw new IllegalArgumentException(ex.getMessage(), ex.getCause());
        }
    }

    @Override
    public void start(@NotNull String name, @NotNull BotConfigSupport config) {
        try {
            ChatBotSupervisor.INSTANCE.start(name, config.unwrap());
        } catch (BotCreationException ex) {
            throw new IllegalArgumentException(ex.getMessage(), ex.getCause());
        }
    }

    @Override
    public void stop(@NotNull String name) throws InterruptedException {
        ChatBotSupervisor.INSTANCE.stop(name);
    }

    @Override
    public void stop() throws InterruptedException {
        ChatBotSupervisor.INSTANCE.stop();
    }

    @Override
    public @NotNull Map<@NotNull String, @NotNull BotConfigSupport> getBots() {
        var out = new TreeMap<String, BotConfigSupport>();
        ChatBotSupervisor.INSTANCE.getBots().forEach((key, value) -> out.put(key, new BotConfigSupport(value)));
        return Collections.unmodifiableMap(out);
    }

    @Override
    public @NotNull SortedSet<@NotNull String> getBotImplementations() {
        return Collections.unmodifiableSortedSet(
                ChatBotFactory.implementations().stream()
                        .map(Class::getCanonicalName)
                        .collect(Collectors.toCollection(TreeSet::new))
        );
    }
}
